<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

class ConnectPay extends PaymentModule
{
    public function __construct()
    {
        $this->name = 'connectpay';
        $this->version = '1.2.0';
        $this->module_key = '4d962d7112fe7bd6da49a1b137a9b483';
        $this->author = 'ConnectPay';
        $this->tab = 'payments_gateways';

        parent::__construct();
        $this->displayName = $this->l('ConnectPay');
        $this->description = $this->l('ConnectPay Accept Payments module allows collecting all types of payments. Easily you could manage everything with one ConnectPay account.');
        $this->ps_versions_compliancy = [
            'min' => '1.6.1.20',
            'max' => _PS_VERSION_,
        ];

        $this->autoLoad();
    }

    /**
     * Gets service that is defined by module container.
     *
     * @param string $serviceName
     *
     * @return object|null
     */
    public function getService(string $serviceName)
    {
        $container = new League\Container\Container;
        $container->delegate(
            (new \League\Container\ReflectionContainer())->cacheResolutions()
        );

        $container->addServiceProvider(new Invertus\ConnectPay\Infrastructure\ServiceProvider\BaseServiceProvider());
        return $container->get($serviceName);
    }

    /**
     * {@inheritdoc}
     */
    public function getTabs(): array
    {
        $tabConfig = new Invertus\ConnectPay\Config\TabConfig();

        return $tabConfig->getTabs();
    }

    /**
     * {@inheritdoc}
     */
    public function getContent(): void
    {
        $redirectLink = $this->context->link->getAdminLink(Invertus\ConnectPay\Config\ConnectPayConfig::OAUTH_CONTROLLER);
        Tools::redirectAdmin($redirectLink);
    }

    public function install(): bool
    {
        $install = parent::install();

        if (!$install) {
            return false;
        }

        if (!\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isSupportedPhpVersion()) {
            $this->_errors[] = sprintf(
                $this->l('Your PHP version is not supported. Minimum supported version is %s'),
                [
                    '7.1',
                ]
            );

            return false;
        }

        /** @var Invertus\ConnectPay\Install\Installer $installer */
        $installer = $this->getService(Invertus\ConnectPay\Install\Installer::class);

        try {
            $installer->init();

            return true;
        } catch (Invertus\ConnectPay\Infrastructure\Bootstrap\ApplicationBootstrapException $exception) {
            $this->_errors[] = $this->l('An unknown error occurred. Please check system logs or contact support');
        }

        return false;
    }

    public function uninstall(): bool
    {
        $uninstall = parent::uninstall();

        if (!$uninstall) {
            return false;
        }

        /** @var Invertus\ConnectPay\Install\Uninstaller $uninstaller */
        $uninstaller = $this->getService(Invertus\ConnectPay\Install\Uninstaller::class);

        try {
            $uninstaller->init();

            return true;
        } catch (Invertus\ConnectPay\Infrastructure\Bootstrap\ApplicationBootstrapException $exception) {
            $this->_errors[] = $this->l('An unknown error occurred. Please check system logs or contact support');

            return false;
        }
    }


    public function hookActionDispatcherBefore(): void
    {
        $this->autoLoad();
    }

    /**
     * Autoload's project files from /src directory
     */
    private function autoLoad(): void
    {
        $autoLoadPath = $this->getLocalPath() . 'vendor/autoload.php';

        require_once $autoLoadPath;
    }

    /**
     * Display this module as a payment option during the checkout
     * @return array|void
     */
    public function hookPaymentOptions(array $params)
    {
        if (\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
            return [];
        }

        $currency = new Currency($this->context->cart->id_currency);
        if (!in_array($currency->iso_code, \Invertus\ConnectPay\Config\ConnectPayConfig::MANDATORY_PAYMENT_CURRENCIES, true)) {
            return [];
        }

        $this->context->controller->addJS("{$this->getPathUri()}views/js/front/payment.js");

        /**
         * @var \Invertus\ConnectPay\Presentation\Presenter\PaymentOptionsPresenter $paymentOptionPresenter
         */
        $paymentOptionPresenter = $this->getService(
            \Invertus\ConnectPay\Presentation\Presenter\PaymentOptionsPresenter::class
        );

        // Set terms of use.
        $termsOfUseUrl = \Invertus\ConnectPay\Config\ConnectPayConfig::TERMS_OF_USE_LINK;
        $privacyPolicyUrl = \Invertus\ConnectPay\Config\ConnectPayConfig::PRIVACY_POLICY_LINK;
        $shopIso = $this->context->language->iso_code;
        $termsOfUseLinkByLang = (isset($termsOfUseUrl[$shopIso])) ? $termsOfUseUrl[$shopIso] : $termsOfUseUrl['default'];
        $privacyPolicyLinkByLang = (isset($privacyPolicyUrl[$shopIso])) ? $privacyPolicyUrl[$shopIso] : $privacyPolicyUrl['default'];

        $this->context->smarty->assign(
            [
                'termsOfUseLink' => $termsOfUseLinkByLang,
                'privacyPolicyLink' => $privacyPolicyLinkByLang,
            ]
        );

        return $paymentOptionPresenter->present();
    }

    /**
     * PS 1.6 display payment options.
     */
    public function hookDisplayPayment(array $params): string
    {
        if (!\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
            return '';
        }

        $currency = new Currency($this->context->cart->id_currency);
        if (!in_array($currency->iso_code, \Invertus\ConnectPay\Config\ConnectPayConfig::MANDATORY_PAYMENT_CURRENCIES, true)) {
            return '';
        }

        /**
         * @var \Invertus\ConnectPay\Presentation\Presenter\PaymentOptionsPresenter $paymentOptionPresenter
         */
        $paymentOptionPresenter = $this->getService(
            \Invertus\ConnectPay\Presentation\Presenter\PaymentOptionsPresenter::class
        );

        $this->context->controller->addJS("{$this->getPathUri()}views/js/front/payment.js");

        // Set terms of use.
        $termsOfUseUrl = \Invertus\ConnectPay\Config\ConnectPayConfig::TERMS_OF_USE_LINK;
        $privacyPolicyUrl = \Invertus\ConnectPay\Config\ConnectPayConfig::PRIVACY_POLICY_LINK;
        $shopIso = $this->context->language->iso_code;
        $termsOfUseLinkByLang = (isset($termsOfUseUrl[$shopIso])) ? $termsOfUseUrl[$shopIso] : $termsOfUseUrl['default'];
        $privacyPolicyLinkByLang = (isset($privacyPolicyUrl[$shopIso])) ? $privacyPolicyUrl[$shopIso] : $privacyPolicyUrl['default'];
        $providers = $paymentOptionPresenter->present();

        $this->context->smarty->assign(
            [
                'termsOfUseLink' => $termsOfUseLinkByLang,
                'privacyPolicyLink' => $privacyPolicyLinkByLang,
                'providersMain' => $providers,
            ]
        );

        return $this->display(__FILE__, 'front/paymentsPs16.tpl');
    }

    /**
     * Display a message in the paymentReturn hook
     */
    public function hookPaymentReturn(array $params): string
    {
        return '';
    }

    /**
     * Display a message in the paymentReturn hook
     */
    public function hookDisplayPaymentReturn($params)
    {
        return '';
    }

    public function hookActionFrontControllerSetMedia(): void
    {
        if ($this->context->controller instanceof OrderController) {
            $this->context->controller->addJS("{$this->getPathUri()}views/js/front/payment.js");

            if (\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
                $this->context->controller->addCSS("{$this->getPathUri()}views/css/front/paymentForm.css");
            } else {
                $this->context->controller->addCSS("{$this->getPathUri()}views/css/front/checkout.css");
            }
        }

        $this->displayErrorMessages();
    }

    public function hookDisplayOrderConfirmation(): string
    {
        if (!\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
            return '';
        }

        $moduleId = (int) Tools::getValue('id_module');
        $connectPayInstance = Module::getInstanceById($moduleId);

        if ($this !== $connectPayInstance) {
            return '';
        }

        $secureKey = Tools::getValue('key');
        $cartId = Tools::getValue('id_cart');
        $cart = new Cart($cartId);
        $orderId = (int) Tools::getValue('id_order');
        $order = new Order($orderId);

        if ((int) $order->id_customer !== $this->context->customer->id ||
            $cart->secure_key !== $secureKey
        ) {
            return '';
        }

        return $this->context->smarty->fetch(
            "{$this->getLocalPath()}/views/templates/hook/ps16_order_confirmation.tpl",
            [
                'reference' => $order->reference,
            ]
        );
    }


    /**
     * Display error messages in frontend ps16
     */
    private function displayErrorMessages(): void
    {
        if (!\Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isPsVersionLowerThan1700()) {
            return;
        }
        /**
         * @var \Invertus\ConnectPay\Infrastructure\Notification\ErrorNotification $errorNotification
         */
        $errorNotification = $this->getService(
            \Invertus\ConnectPay\Infrastructure\Notification\ErrorNotification::class
        );

        $this->context->controller->errors = $errorNotification->getMessages();
    }

    private function displayInAdminOrderPage(array $params)
    {
        $orderId = $params['id_order'];
        $order = new Order($orderId);

        if ($order->module !== $this->name) {
            return;
        }

        $connectPayOrder = [
            'connectPayTransactionId' => $this->l('ConnectPay transaction ID not found.')
        ];

        /** @var \Invertus\ConnectPay\Payment\Repository\ConnectPayOrderRepository $connectPayOrderRepository */
        $connectPayOrderRepository =
            $this->getService(Invertus\ConnectPay\Payment\Repository\ConnectPayOrderRepository::class);
        $connectPayTransactionId = $connectPayOrderRepository->getConnectPayPaymentIdByOrderId($orderId);

        if (!empty($connectPayTransactionId)) {
            $connectPayOrder = [
                'connectPayTransactionId' => $connectPayTransactionId
            ];
        }

        $this->context->smarty->assign($connectPayOrder);

        return $this->context->smarty->fetch(
            $this->getLocalPath() . 'views/templates/hook/admin/connectpay_order.tpl'
        );
    }

    public function hookDisplayAdminOrderTabContent(array $params)
    {
        $isVersionAbove177 = \Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isVersionAbove177();

        return !$isVersionAbove177 ? false : $this->displayInAdminOrderPage($params);
    }


    public function hookDisplayAdminOrder(array $params)
    {
        $isVersionAbove177 = \Invertus\ConnectPay\Infrastructure\Utility\VersionUtility::isVersionAbove177();

        return $isVersionAbove177 ? false : $this->displayInAdminOrderPage($params);
    }
}
