<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\Notification\ErrorNotification;
use Invertus\ConnectPay\Adapter\Tools;
use Invertus\ConnectPay\Exception\ConnectPayException;
use Invertus\ConnectPay\Infrastructure\Utility\VersionUtility;
use Invertus\ConnectPay\OAuth\PrivateKey\SavePrivateKeyHandler;

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminConnectPayOAuthController extends ModuleAdminController
{
    private const PRIVATE_KEY_FORMAT = [
        'application/x-x509-ca-cert',
        'application/x-x509-user-cert',
        'application/x-pkcs7-crl',
        'application/x-pem-file',
        'application/x-pkcs12',
        'application/x-pkcs7-certificates',
        'application/x-pkcs7-certreqresp',
        'application/pkix-cert',
        'application/octet-stream',
    ];

    /** @var ConnectPay */
    public $module;

    private $tools;

    public function __construct()
    {
        $this->bootstrap = true;
        parent::__construct();
        $this->override_folder = 'field-option-settings/';
        $this->tpl_folder = 'field-option-settings/';
    }

    public function init()
    {
        /** @var Tools tools */
        $this->tools = $this->module->getService(Tools::class);

        parent::init();

        $this->context->smarty->assign([
            'connectpay' => [
                'update_order_state_cronjob_link' => $this->context->link->getModuleLink(
                    $this->module->name,
                    'updateOrderState',
                    [
                        'token' => $this->tools->encrypt($this->module->name),
                    ]
                ),
                'manual_uri' => ConnectPayConfig::MODULE_URI,
            ],
        ]);

        $this->initConnectFieldForm();
    }

    public function initContent(): void
    {
        $this->content .= $this->context->smarty->fetch(
            "{$this->module->getLocalPath()}views/templates/admin/content-wrapper-start.tpl"
        );

        $this->content .= $this->context->smarty->fetch(
            "{$this->module->getLocalPath()}views/templates/admin/intro.tpl",
            [
                'connectpay' => [
                    'logoPath' => "{$this->module->getPathUri()}views/img/connectpay_logo.png",
                    'lang_iso' => $this->context->language->iso_code,
                ],
            ]
        );

        parent::initContent();

        $this->content .= $this->context->smarty->fetch(
            "{$this->module->getLocalPath()}views/templates/admin/content-wrapper-end.tpl"
        );
    }

    private function initConnectFieldForm(): void
    {
        $this->fields_options = [
            'connect_configurations' => [
                'title' => '',
                'icon' => 'icon-settings',
                'hide_multishop_checkbox' => true,
                'fields' => [
                    'module_setup' => [
                        'type' => 'desc',
                        'class' => 'col-lg-12',
                        'default' => '0',
                        'template' => 'field-module-set-up.tpl',
                        'no_multishop_checkbox' => true,
                    ],
                    'connectpay_values_help' => [
                        'type' => 'desc',
                        'class' => 'col-lg-12',
                        'default' => '0',
                        'template' => 'field-connectpay-values-help.tpl',
                        'no_multishop_checkbox' => true,
                    ],
                    ConnectPayConfig::CLIENT_KEY => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Client ID'),
                        'type' => 'text',
                        'no_multishop_checkbox' => true,
                        'required' => true,
                    ],
                    ConnectPayConfig::CLIENT_SECRET => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Client Secret'),
                        'type' => 'text',
                        'no_multishop_checkbox' => true,
                        'required' => true,
                    ],
                    ConnectPayConfig::BRAND_ID => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Brand ID'),
                        'type' => 'text',
                        'no_multishop_checkbox' => true,
                        'required' => true,
                    ],
                    'privat_key_help' => [
                        'type' => 'desc',
                        'class' => 'col-lg-12',
                        'default' => '0',
                        'template' => 'field-private-key-help.tpl',
                        'no_multishop_checkbox' => true,
                    ],
                    ConnectPayConfig::CARD => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('CARD'),
                        'type' => 'bool',
                        'no_multishop_checkbox' => true,
                        'required' => false,
                    ],
                    ConnectPayConfig::PID => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Bank transfer'),
                        'type' => 'bool',
                        'no_multishop_checkbox' => true,
                        'required' => false,
                    ],
                ],
                'buttons' => [
                    'save_and_connect' => [
                        'title' => $this->l('Connect'),
                        'icon' => 'process-icon-save',
                        'class' => 'btn btn-default pull-left',
                        'type' => 'submit',
                    ],
                ],
            ],
        ];

        if (VersionUtility::isVersionAbove170()) {
            $this->fields_options['connect_configurations']['fields'][ConnectPayConfig::FAILED_ORDER_CART_SAVE] = [
                'form_group_class' => 'login-inputs',
                'title' => $this->l('Save failed cart data'),
                'type' => 'bool',
                'no_multishop_checkbox' => true,
                'required' => false,
                'hint' => $this->l(
                    'If you enable this feature, on failed payment case first order is Canceled returning goods to stock.
                     Customer can pay again with different order created without loosing cart data.'
                ),
            ];
        }

        $this->fields_options['connect_configurations']['fields'][ConnectPayConfig::LIVE] = [
            'form_group_class' => 'login-inputs',
            'title' => $this->l('IS LIVE?'),
            'type' => 'bool',
            'no_multishop_checkbox' => true,
            'required' => false,
        ];

        $this->fields_options['connect_configurations']['fields']['cron_config_help'] = [
            'type' => 'desc',
            'class' => 'col-lg-12',
            'default' => '0',
            'template' => 'cron-config-help.tpl',
            'no_multishop_checkbox' => true,
        ];
    }

    public function postProcess(): void
    {
        parent::postProcess();
        /** @var Tools tools */
        $tools = $this->module->getService(Tools::class);
        /** @var ErrorNotification $errorNotification */
        $errorNotification = $this->module->getService(ErrorNotification::class);

        if ($tools->isSubmit('submitOptionsconfiguration')) {
            if (empty($tools->getValue(ConnectPayConfig::CLIENT_KEY))) {
                $this->errors[] = $this->l('Please provide Client ID to connect.');
                $this->confirmations = []; //NOTE to clear "Settings updated message"
            }

            if (empty($tools->getValue(ConnectPayConfig::CLIENT_SECRET))) {
                $this->errors[] = $this->l('Please provide Client Secret to connect.');
                $this->confirmations = [];
            }

            if (
                (!empty($_FILES['privateKey']['name']) ||
                $_FILES['privateKey']['size'] !== 0) &&
                !in_array($_FILES['privateKey']['type'], self::PRIVATE_KEY_FORMAT)
            ) {
                $this->errors[] = $this->l('Incorrect Private key file type.');
                $this->confirmations = [];
            }

            if (!empty($this->errors)) {
                return;
            }

            /** @var SavePrivateKeyHandler $savePrivateKeyHandler */
            $savePrivateKeyHandler = $this->module->getService(SavePrivateKeyHandler::class);

            // Save Private Key
            try {
                if (!empty($_FILES['privateKey']['name']) || $_FILES['privateKey']['size'] !== 0) {
                    $savePrivateKeyHandler->handle($_FILES['privateKey'], $this->module);
                }

            } catch (ConnectPayException $e) {
                $errorMessage = sprintf(
                    $this->l('Error occurred during Private key saving. Error code: %s , message: %s'),
                    $e->getCode(),
                    $e->getMessage()
                );
                $errorNotification->setMessage($errorMessage);
                $this->errors[] = $errorMessage;
                $this->confirmations = [];
                return;
            }
        }
    }

    public function setMedia($isNewTheme = false): void
    {
        parent::setMedia();

        $this->addCSS("{$this->module->getPathUri()}views/css/admin/admin.css");
        $this->addJS("{$this->module->getPathUri()}/views/js/admin/admin.js");
    }
}
