<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Adapter\Tools;
use Invertus\ConnectPay\Infrastructure\Context\GlobalShopContextInterface;
use Invertus\ConnectPay\Payment\API\ProvidersApiRequest;
use Invertus\ConnectPay\Payment\Service\ProviderCreatorService;
use Invertus\ConnectPay\Payment\Repository\ConnectPayProviderRepository;

if (!defined('_PS_VERSION_')) {
    exit;
}

class AdminConnectPayPaymentsController extends ModuleAdminController
{
    /** @var ConnectPay */
    public $module;

    public function __construct()
    {
        $this->bootstrap = true;
        parent::__construct();
        $this->override_folder = 'field-option-settings/';
        $this->tpl_folder = 'field-option-settings/';
    }

    public function init()
    {
        parent::init();

        $this->initConnectFieldForm();
    }

    private function initConnectFieldForm(): void
    {
        $selectorList = [
            [
                'country_code' => '',
                'name' => $this->l('Select country')
            ],
        ];
        $selectorList = array_merge_recursive($selectorList, ConnectPayConfig::getProvidersSelectorOptions());

        $this->fields_options = [
            'connect_configurations' => [
                'title' => $this->l('Display providers in FO configuration'),
                'icon' => 'icon-settings',
                'hide_multishop_checkbox' => true,
                'fields' => [
                    'display_all_banks_description' => [
                        'type' => 'desc',
                        'class' => 'col-lg-12',
                        'default' => '0',
                        'template' => 'field-display-all-banks-description.tpl',
                        'no_multishop_checkbox' => true,
                    ],
                    ConnectPayConfig::DISPLAY_ALL_BANKS => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Display All Banks'),
                        'type' => 'bool',
                        'no_multishop_checkbox' => true,
                        'required' => true,
                    ],
                ],
                'buttons' => [
                    'save_and_connect' => [
                        'title' => $this->l('Save'),
                        'icon' => 'process-icon-save',
                        'class' => 'btn btn-default',
                        'type' => 'submit',
                    ],
                ],
            ],
            'providers' => [
                'title' => $this->l('Payment Providers'),
                'icon' => 'icon-settings',
                'fields' => [
                    ConnectPayConfig::SELECT_PROVIDERS_PLACEHOLDER => [
                        'form_group_class' => 'login-inputs',
                        'title' => $this->l('Select provider country'),
                        'type' => 'select',
                        'list' => $selectorList,
                        'identifier' => 'country_code',
                        'required' => false,
                    ],
                ],
                'buttons' => [
                    'save_and_connect' => [
                        'title' => $this->l('Load Providers'),
                        'icon' => 'process-icon-save',
                        'class' => 'btn btn-default',
                        'type' => 'submit',
                    ],
                ],
            ],
        ];
    }

    public function postProcess(): void
    {
        parent::postProcess();

        /** @var Tools $tools */
        $tools = $this->module->getService(Tools::class);
        /** @var Configuration $configuration */
        $configuration = $this->module->getService(Configuration::class);
        /** @var GlobalShopContextInterface $prestaShopContext */
        $prestaShopContext = $this->module->getService(GlobalShopContextInterface::class);

        if ($tools->isSubmit('submitOptionsconfiguration')) {
            $configuration->set(
                ConnectPayConfig::DISPLAY_ALL_BANKS,
                $tools->getValue(ConnectPayConfig::DISPLAY_ALL_BANKS),
                $prestaShopContext->getShopId()
            );

            // Load providers.
            if (!empty($tools->getValue(ConnectPayConfig::SELECT_PROVIDERS_PLACEHOLDER))) {
                /** @var ProvidersApiRequest $providersApi */
                $providersApi = $this->module->getService(ProvidersApiRequest::class);

                $countryCode = $tools->getValue(ConnectPayConfig::SELECT_PROVIDERS_PLACEHOLDER);
                $providers = $providersApi->getProviders($countryCode);

                if ($providers === null) {
                    $this->errors[] = $this->l('Something went wrong. Please check module logs.');
                    $this->confirmations = [];
                    return;
                }

                $banks = [];
                // Display providers.
                foreach ($providers->providers as $provider) {

                    $isActive = ConnectPayProviderRepository::isActive($provider->id);
                    $logo = $provider->presentation->logo ?? "{$this->module->getPathUri()}views/img/default_provider_img.svg";

                    $banks[] = [
                        'isActive' => $isActive,
                        'name' => $provider->name,
                        'logo' => $logo,
                        'id' => $provider->id,
                    ];
                }

                $this->context->smarty->assign('providers', $banks);

                $this->fields_options['providers']['fields']['banks'] = [
                    'type' => 'desc',
                    'class' => 'col-lg-12',
                    'default' => '0',
                    'template' => 'field-display-all-banks.tpl',
                ];
            }
        }
    }

    public function setMedia($isNewTheme = false): void
    {
        parent::setMedia();

        $this->addCSS("{$this->module->getPathUri()}views/css/admin/admin.css");
        $this->addJS("{$this->module->getPathUri()}/views/js/admin/admin.js");
    }

    public function ajaxProcessUpdateProvider(): void
    {
        /** @var ProviderCreatorService $providerCreatorService */
        $providerCreatorService = $this->module->getService(ProviderCreatorService::class);
        /** @var Tools $tools */
        $tools = $this->module->getService(Tools::class);

        if ($tools->getValue('active') === 'true') {
            $isActive = true;
        } else {
            $isActive = false;
        }

        $providerCreatorService->updateProvider(
            (string) $tools->getValue('providerCode'),
            (bool) $isActive
        );
    }
}
