<?php
/**
 * NOTICE OF LICENSE
 *
 * @author    Connectpay  <https://connectpay.com/
 * @copyright Copyright (c) permanent, ConnectPay Inc.
 * @license   Apache License
 *
 * @see       /LICENSE
 *
 *  International Registered Trademark & Property of ConnectPay Inc.
 */

declare(strict_types=1);

namespace Invertus\ConnectPay\Infrastructure\Bootstrap;

use Invertus\ConnectPay\Adapter\Configuration;
use Invertus\ConnectPay\Config\ConnectPayConfig;
use Invertus\ConnectPay\Infrastructure\Adapter\Language;
use Invertus\ConnectPay\Infrastructure\Adapter\ModuleFactory;
use Invertus\ConnectPay\Infrastructure\Adapter\Shop;
use Exception;
use OrderState;
use Validate;

class OrderStateInstaller implements InstallerInterface
{
    private $moduleFactory;
    private $configuration;
    private $shop;
    private $language;

    public function __construct(
        ModuleFactory $moduleFactory,
        Configuration $configuration,
        Shop $shop,
        Language $language
    ) {
        $this->moduleFactory = $moduleFactory;
        $this->configuration = $configuration;
        $this->shop = $shop;
        $this->language = $language;
    }

    /**
     * @throws ApplicationBootstrapException
     */
    public function init(): void
    {
        $shopIds = $this->shop->getContextListShopId();

		$colorCode = [
			'blue' => '#3498D8',
			'green' => '#01B887',
			'red' => '#8f0821',
		];

        foreach ($shopIds as $shopId) {
            $this->createState(
                (int) $shopId,
                ConnectPayConfig::ORDER_STATUS_AWAITING_PAYMENT,
	            $colorCode['blue'],
                'Awaiting ConnectPay Payment'
            );
            $this->createState(
                (int) $shopId,
                ConnectPayConfig::ORDER_STATUS_APPROVED,
	            $colorCode['green'],
                'ConnectPay Payment Completed'
            );
            $this->createState(
                (int) $shopId,
                ConnectPayConfig::ORDER_STATUS_FAILED,
	            $colorCode['red'],
                'ConnectPay Payment Failed'
            );
        }
    }

    /**
     * @param int $shopId
     * @throws ApplicationBootstrapException
     */
    private function createState(int $shopId, string $status, string $color, string $name): void
    {
        $existingStateId = (int) $this->configuration->get($status, $shopId);
        $orderState = new OrderState($existingStateId, null, $shopId);

        // if state already existed we wont install new one.
        if (Validate::isLoadedObject($orderState)) {
            return;
        }

        $orderState->send_email = false;
        $orderState->color = $color;
        $orderState->hidden = false;
        $orderState->delivery = false;
        $orderState->logable = false;
        $orderState->invoice = false;
        $orderState->unremovable = false;
        $orderState->module_name = $this->moduleFactory->getModule()->name;

        $languages = $this->language->getAllLanguages();
        foreach ($languages as $language) {
            $orderState->name[$language['id_lang']] = $name;
        }

        try {
            $orderState->add();
        } catch (Exception $e) {
            throw new ApplicationBootstrapException(
                'Failed to install order state %s',
                $orderState->name[$this->language->getDefaultLanguageId()]
            );
        }

        $this->configuration->set(
            $status,
            (int) $orderState->id,
            (int) $shopId
        );
    }
}
